#ifndef L2DEFOBJ
#define L2DEFOBJ

#include <vector>
#include "L2FileHndl.h"
#include "L2DefTile.h"
#include "L2GnrlGfx.h"

using namespace std;

#define L2OB 6 //(object information)
#define L2BF 7 //(animation-frame data)
#define L2BA 8 //(animation-frame data pointers)
#define L2BI 9 //(animation-frame arrangement pointers)

#define BORDER_NONE 0
#define BORDER_NORMAL 1
#define BORDER_SPECIAL 2

struct ObjectComponent
{
    bool NoDrawFlag;    //Sometimes Special Gfx of unknown Format
    //do not draw if (third to last) == 0x00, or (0xe0 & last) != 0 && (second to last) == 0x00
    int GfxID;
    int xOffset, yOffset;
    bool HorizontalExt, VerticalExt;
    bool RelativeXPos, RelativeYPos;
};

class CObjectSingle {
public:
    void CreateObject(FILE* file);
    void GetSize();
    void DrawObject(int xPos, int yPos, int xExtension, int yExtension, int BorderMode);
public:
    int xMin, yMin, xMax, yMax, xPlus, yPlus; //necessary for Border
    bool SwingThingie;
    int ComponentAmount;
    ObjectComponent* ComponentList;
};

class CDefObject {
public:
    void CreateObjects(FILE* StyleFile);
    void DrawObject(int x, int y, int xExtn, int yExtn, int ID, int BorderMode);
public:
    int ObjectAmount;
    int ComponentAmount;
    vector<CTileSingle*> ObjectComponentList;
    vector<CObjectSingle*> ObjectList;
};

CDefObject* ObjectData = NULL;

void CObjectSingle::CreateObject(FILE* file)
{
    ComponentAmount = fgetc(file);
    ComponentList = new ObjectComponent[ComponentAmount];
    fseek(file, 0x13, SEEK_CUR);
    xMin=0; yMin=0; xMax=0; yMax=0; xPlus=0; yPlus=0;
    SwingThingie = false;
    for(int i=0; i<ComponentAmount; i++)
    {
        ComponentList[i].VerticalExt = false;
        ComponentList[i].HorizontalExt = false;
        ComponentList[i].RelativeYPos = false;
        ComponentList[i].RelativeXPos = false;
        ComponentList[i].NoDrawFlag = false;
        fseek(file, 0x01, SEEK_CUR);
        int byte=fgetc(file);
        if(byte & 0x10) ComponentList[i].VerticalExt = true;
        if(byte & 0x20) ComponentList[i].HorizontalExt = true;
        if(byte & 0x40) ComponentList[i].RelativeYPos = true;
        if(byte & 0x80) ComponentList[i].RelativeXPos = true;
        ComponentList[i].xOffset = ReadValue(file, 2, 'l');
        ComponentList[i].yOffset = ReadValue(file, 2, 'l');
        fseek(file, 0x03, SEEK_CUR);
        if(fgetc(file)==0x00) ComponentList[i].NoDrawFlag = true; //provisional detection
        ComponentList[i].GfxID = fgetc(file);
        byte = fgetc(file);
        if((byte & 0xe0) != 0x00) if(ComponentList[i].GfxID == 0x00) ComponentList[i].NoDrawFlag = true;
        if(byte == 0xe0) SwingThingie = true;
    }
      if(ComponentList[0].RelativeXPos) xMin=ComponentList[0].xOffset;
      if(ComponentList[ComponentAmount-1].RelativeXPos) xMax+=ComponentList[ComponentAmount].xOffset;
}

void CObjectSingle::GetSize()
{
    xPlus=0; yPlus=0;
    int xSizeAtMax=0, ySizeAtMax=0;
    int xOff=0, yOff=0; /*ComponentList[0].xOffset*/
    for(int i=0; i<ComponentAmount; i++)
    {
        if(ComponentList[i].HorizontalExt) xPlus += 16*ObjectData->ObjectComponentList[ComponentList[i].GfxID]->xSize;
        if(ComponentList[i].VerticalExt) yPlus += 8*ObjectData->ObjectComponentList[ComponentList[i].GfxID]->ySize;
        
        if(ComponentList[i].RelativeXPos) xOff += ComponentList[i].xOffset;
        else xOff=ComponentList[i].xOffset;
        if(i==0 || xOff + 16*ObjectData->ObjectComponentList[ComponentList[i].GfxID]->xSize> xMax + xSizeAtMax) {xMax = xOff; xSizeAtMax = 16*ObjectData->ObjectComponentList[ComponentList[i].GfxID]->xSize;}
        if(i==0 || xOff < xMin) xMin = xOff;
        
        if(!ComponentList[i].RelativeYPos) yOff=ComponentList[i].yOffset;
        if(ComponentList[i].RelativeYPos && SwingThingie) yOff += ComponentList[i].yOffset;
        if(i==0 || yOff + 8*ObjectData->ObjectComponentList[ComponentList[i].GfxID]->ySize > yMax + ySizeAtMax) {yMax = yOff; ySizeAtMax = 8*ObjectData->ObjectComponentList[ComponentList[i].GfxID]->ySize;}
        if(i==0 || yOff < yMin) yMin = yOff;        
        //DrawTileSingle(xPos + xOff, yPos + yOff);
        if(ComponentList[i].RelativeYPos && !SwingThingie) yOff += ComponentList[i].yOffset;
    }
    xMax += xSizeAtMax;
    yMax += ySizeAtMax;
}

void CObjectSingle::DrawObject(int xPos, int yPos, int xExtension, int yExtension, int BorderMode)
{
    int xOff=0, yOff=0; /*ComponentList[0].xOffset*/
    int xExt, yExt;
    int yOffTemp;
    for(int i=0; i<ComponentAmount; i++)
    {
        if(ComponentList[i].HorizontalExt) xExt = xExtension;
        else xExt=0;
        if(ComponentList[i].VerticalExt) yExt = yExtension;
        else yExt=0;
        
        for(int x=0; x<=xExt; x++)
        {
            if(ComponentList[i].RelativeXPos) xOff += ComponentList[i].xOffset;
            else xOff=ComponentList[i].xOffset;
            yOffTemp = yOff;
            for(int y=0; y<=yExt; y++)
            {
                if(!ComponentList[i].RelativeYPos) yOffTemp=ComponentList[i].yOffset;
                if(ComponentList[i].RelativeYPos && SwingThingie) yOffTemp += ComponentList[i].yOffset;
                if(!ComponentList[i].NoDrawFlag)
                    ObjectData->ObjectComponentList[ComponentList[i].GfxID]->DrawTileSingle(xPos + xOff, yPos + yOffTemp, false);
                if(ComponentList[i].RelativeYPos && !SwingThingie) yOffTemp += ComponentList[i].yOffset;
            }
        }
        yOff = yOffTemp;
        /*if(ComponentList[i].HorizontalExt && xExtension)
        {
            if(ComponentList[i].VerticalExt && yExtension)
            {
                for(int x = xOff; x < xOff + (ComponentList[i].xOffset)*(xExtension+1); x += ComponentList[i].xOffset)
                    if(!ComponentList[i].NoDrawFlag)
                        ObjectData->ObjectComponentList[ComponentList[i].GfxID]->DrawTileSingle(xPos + x, yPos + yOff);
                yOff += ComponentList[i].yOffset;
                yExtension--;
                i--;
            }
            else
            {
                xOff += ComponentList[i].xOffset;
                if(!ComponentList[i].NoDrawFlag)
                    ObjectData->ObjectComponentList[ComponentList[i].GfxID]->DrawTileSingle(xPos + xOff, yPos + yOff);
                xExtension--;
                i--;
            }
        }
        else if(ComponentList[i].VerticalExt && yExtension)
        {
            if(!ComponentList[i].NoDrawFlag)
                ObjectData->ObjectComponentList[ComponentList[i].GfxID]->DrawTileSingle(xPos + xOff, yPos + yOff);
            yOff += ComponentList[i].yOffset;
            yExtension--;
            i--;
        }
        else
        {
            xOff += ComponentList[i].xOffset;
            if(!ComponentList[i].NoDrawFlag)
                ObjectData->ObjectComponentList[ComponentList[i].GfxID]->DrawTileSingle(xPos + xOff, yPos + yOff);
            yOff += ComponentList[i].yOffset;
        }*/
    }

    if(BorderMode)
    {
        if(BorderMode == BORDER_NORMAL) glColor3f(1.0,1.0,1.0);
        else glColor3f(1.0,0.0,0.0);
        DrawRectCoords(xPos+xMin, yPos+yMin, xPos+xMax+xExtension*xPlus-1, yPos+yMax+yExtension*yPlus-1);
    }
}

void CDefObject::CreateObjects(FILE* StyleFile)
{    
    CObjectSingle* SingleObject;
    CTileSingle* SingleComponent;
    
    GetSectionOffset(StyleFile, L2OB);
    fseek(StyleFile, 0x08, SEEK_CUR);
    ObjectAmount = ReadValue(StyleFile, 2, 'l');
    
    FILE* debug = NULL;//fopen("debug.txt", "a+");
    /* Retrieve Object Information */
    for(int i=0; i<ObjectAmount; i++)
    {
        //fprintf(debug, "%x, ", ftell(StyleFile));
        SingleObject = new CObjectSingle;
        SingleObject->CreateObject(StyleFile);
        ObjectList.push_back(SingleObject);
    }
        
    /* Retrieve Data of Object Components */
    int offset = GetSectionOffset(StyleFile, L2BI);
    fseek(StyleFile, 0x08, SEEK_CUR);
    ComponentAmount = ReadValue(StyleFile, 2, 'l');
    for(int i=0; i<ComponentAmount; i++)
    {
        fseek(StyleFile, offset+0x0a+2*i, SEEK_SET);
        //fprintf(debug, "%x\n", ftell(StyleFile));
        SingleComponent = new CTileSingle;
        SingleComponent->GenerateObject(StyleFile, i, debug);
        ObjectComponentList.push_back(SingleComponent);
    }
    
    /* Determine Size of Objects */
    for(int i=0; i<ObjectAmount; i++) ObjectList[i]->GetSize();
    //fclose(debug);
}

void CDefObject::DrawObject(int x, int y, int xExtn, int yExtn, int ID, int BorderMode)
{
    if(ID < ObjectAmount) ObjectList[ID]->DrawObject(x, y, xExtn, yExtn, BorderMode);
}

#endif
