#ifndef L2DEFTILE
#define L2DEFTILE

#include <vector>
#include "L2FileHndl.h"

using namespace std;

#define L2BE 5 //(terrain tile arrangement)
#define L2BF 7 //(animation-frame data)
#define L2BA 8 //(animation-frame data pointers)
#define L2BI 9 //(animation-frame arrangement pointers)


/*
L2CL 0 (palette)
L2SS 1 (special-object sprites)
L2SF 2 (special-object-frame data)
L2SA 3 (special-object-frame data pointers)
L2SI 4 (special-object-frame arrangement pointers)
L2BE 5 (terrain tile arrangement)
L2OB 6 (object information)
L2BF 7 (animation-frame data)
L2BA 8 (animation-frame data pointers)
L2BI 9 (animation-frame arrangement pointers)
L2BL a (8*16 sprites)
L2BS b (preview 2*1 sprites)
*/

class CTileSingle {
public:
    void GenerateTile(FILE* file);
    void GenerateObject(FILE* file, int Index, FILE* debug);
    void DrawTileSingle(int x, int y, bool bgDraw);
public:
    int xSize; int ySize;
    short* SpriteList;
};

class CDefTile {
public:
	void GenerateTiles(FILE* StyleFile);
    void DrawTile(int x, int y, int ID, bool bgDraw);
	//void WriteTileToData();
public:
    int TileAmount;
    vector<CTileSingle*> TileList;
};

CDefTile* TileData = NULL;

void CTileSingle::GenerateTile(FILE* file)
{
    fseek(file, 0x02, SEEK_CUR); //unknown
    xSize=fgetc(file); //xSize
    ySize=fgetc(file); //ySize
    fseek(file, 0x02, SEEK_CUR); //entrySize
    SpriteList = new short[xSize*ySize];
    for(int j=0; j < xSize*ySize; j++) SpriteList[j]=ReadValue(file, 2, 'l');
}

void CTileSingle::GenerateObject(FILE* file, int Index, FILE* debug)
{
    GetSectionOffset(file, L2BI);
    fseek(file, 0x0a + 2*Index, SEEK_CUR);
     //fprintf(debug, " %x\n", ftell(file));
    int AnimationOffset = ReadValue(file, 2, 'l');
    
    GetSectionOffset(file, L2BA);
    fseek(file, 0x0a + AnimationOffset + 0x02, SEEK_CUR);
     //fprintf(debug, " %x\n", ftell(file));
    xSize = ReadValue(file, 2, 'l');
    ySize = ReadValue(file, 2, 'l');
    int FrameOffset = ReadValue(file, 2, 'l');
    
    GetSectionOffset(file, L2BF);
    fseek(file, 0x0a, SEEK_CUR);
    while(FrameOffset>0)
    {
        int Size = ReadValue(file, 2, 'l');
        fseek(file, Size, SEEK_CUR);
        FrameOffset -= Size;
    }
    fseek(file, 0x02, SEEK_CUR);
     //fprintf(debug, " %x\n", ftell(file));
    
    SpriteList = new short[xSize*ySize];
    for(int j=0; j < xSize*ySize; j++) SpriteList[j]=ReadValue(file, 2, 'l');
}

void CTileSingle::DrawTileSingle(int x, int y, bool bgDraw)
{
    int xc, yc;
    for(int i=0; i<xSize*ySize; i++)
    {
        xc = i%xSize;
        yc = i/xSize;
        SpriteData->DrawSprite(x + 16*xc, y + 8*yc, SpriteList[i], bgDraw);
    }
}

void CDefTile::GenerateTiles(FILE* StyleFile)
{
    CTileSingle* SingleTile;
    
    int offset = GetSectionOffset(StyleFile, L2BE);
    fseek(StyleFile, offset+0x08, SEEK_SET);
    TileAmount = ReadValue(StyleFile, 2, 'l');
    
    for(int i=0; i<TileAmount; i++)
    {
        SingleTile = new CTileSingle;
        SingleTile->GenerateTile(StyleFile);
        TileList.push_back(SingleTile);
    }
}

void CDefTile::DrawTile(int x, int y, int ID, bool bgDraw)
{
    if(ID < TileAmount) TileList[ID]->DrawTileSingle(x, y, bgDraw);
}

//list<CObjectDef*> ObjectDefList;
//list<CTileSingle*>::iterator it;

/*
void renderTile(unsigned short tileID, int mouseX, int mouseY)
{
    int spritePos, spriteID;
    int x, y, i=0;
    
    if(gBackgroundDraw) glPixelTransferf(GL_RED_BIAS, 0.2);
    else glPixelTransferf(GL_RED_BIAS, 0.0);

    for(y = mouseY/(8*gZoom); y < mouseY/(8*gZoom) + tileList[tileID].ySize; y++)
    {
        for(x = mouseX/(16*gZoom); x < mouseX/(16*gZoom) + tileList[tileID].xSize; x++)
        {
            spritePos = levelArrangementValue*(y - gTileY) + x - gTileX;
            if(spritePos<1968 && (x-gTileX)<levelArrangementValue && (x-gTileX>=0) && (y-gTileY>=0))
            {
                spriteID = tileData[tileList[tileID].tilePointer + i];
                glRasterPos2i(16*gZoom*x, 8*gZoom*y);
                glCallList(listIndex + spriteID);
                levelMapData[spritePos].spriteID = spriteID;
                levelMapData[spritePos].background = gBackgroundDraw;
                fseek(levelFile, 0x0074 + 4*spritePos, SEEK_SET);
                if(gBackgroundDraw) fputc(0x40, levelFile);
                else fputc(0x00, levelFile);
                fputc(0x00, levelFile);
                fputc(spriteID/0x100, levelFile);
                fputc(spriteID%0x100, levelFile);
            }
            i++;
        }
    }
}*/

#endif
