#ifndef L2FILEHNDL
#define L2FILEHNDL

/* l is little endian, b is big endian */

long ReadValue(FILE* file, short length, char type)
{
    long returnValue = 0;
    for(int i=0; i<length; i++)
    {
        if(type == 'l')
        {
            returnValue += (0x01<<(8*i)) * fgetc(file); //2^8i
        }
        else if(type == 'b')
        {
            returnValue = returnValue*0x100 + fgetc(file);
        }
    }
    if(returnValue & (0x01<<(length*8-1))) returnValue -= (0x01<<(length*8)); //for handling negative values
    return returnValue;
}

void WriteValue(int value, FILE* file, short length, char type)
{
    if(value < 0) value += (0x01<<(length*8)); //for handling negative values
    for(int i=0; i<length; i++)
    {
        if(type == 'l')
        {
            fputc(value & 0xFF, file);
            value = value>>8;
        }
        else if(type == 'b')
        {
            fputc((value & (0xFF<<(8*(length-i-1))))>>(8*(length-i-1)), file);
        }
    }
}

long bigEndian4(unsigned char charBuffer[4])
{
     return (charBuffer[3] + 0x100*charBuffer[2] + 0x10000*charBuffer[1] + 0x1000000*charBuffer[0]);
}

long littleEndian4(unsigned char charBuffer[4])
{
     return (charBuffer[0] + 0x100*charBuffer[1] + 0x10000*charBuffer[2] + 0x1000000*charBuffer[3]);
}

int littleEndian2(unsigned char charBuffer[4])
{
     return (charBuffer[0] + 0x100*charBuffer[1]);
}

int bigEndian2(unsigned char charBuffer[4])
{
     return (charBuffer[1] + 0x100*charBuffer[0]);
}

int GetSectionOffset(FILE* file, int ID)
{
    fseek(file, 0x0C, SEEK_SET);
    unsigned char charBuffer[4];
    long size;

    for(int i=0; i<ID; i++)
    {
         fseek(file, 0x04, SEEK_CUR); //section name
         fread(&charBuffer, sizeof(char), 4, file);
         size = bigEndian4(charBuffer);
         fseek(file, size, SEEK_CUR);
    }
    return ftell(file);
}



#endif
