#ifndef L2LDATA
#define L2LDATA

#include <stdio.h>
#include <vector>
#include "L2DefSprite.h"
#include "L2FileHndl.h"

using namespace std;

#define L2LH 0
#define L2MH 1
#define L2MP 2
#define L2BO 3

struct SSkillData
{
    short int ID;
    char Amount;
};

struct SLMapData
{
    short int SpriteID;
    bool Background;
};

struct SObjectInfo
{
    short int ID,
              xPos,
              yPos,
              xExt,
              yExt;
};

int LevelArrangementList[7]={0x50, 0x40, 0x32, 0x28, 0x20, 0x18, 0x14};

class CLevelData {
public:
    CLevelData(FILE* file) {LevelFile=file; }
    int PointerValidityCheck() {if(LevelFile == NULL) return 1; else return 0;}
	void ReadLevelData();
	void WriteLevelData();
	void SpriteToMap(int x, int y, int ID, bool bg);
	int SpriteFromMap(int x, int y);
	void TileToMap(int x, int y, int ID, bool bg);
    bool CheckObjectUnderCursor(int x, int y, int oID);
 	int ObjectFromMap(int x, int y);
public:
    FILE* LevelFile;
//L2LH section
    char LevelName[24];
    SSkillData SkillDataList[8];
    char TimeMinutes;
    char TimeSeconds;
    //int Style;     //no effect
    int xScreenStart, // hztl. screen start position
        yScreenStart,
        xScreenCutLeft, //cut-off/px from the left
        yScreenCutTop,
        xScreenExtensionRight,
        yScreenExtensionBottom;
    int LemmingsKilledForGold;
    int ReleaseRate;
//L2MH section
    int Style;
    int LevelArrangementValue;
    int LevSpritesPerLine;
//L2MP section
    SLMapData LevelMapData[1971];    
//L2BO section
    vector<SObjectInfo*> ObjectInfoList;
};

CLevelData* LevelData;

void CLevelData::ReadLevelData()
{
    rewind(LevelFile);

    /* Level Arrangement */
    fseek(LevelFile, 0x0068, SEEK_SET);
    LevelArrangementValue = fgetc(LevelFile);
    LevSpritesPerLine = LevelArrangementList[LevelArrangementValue]+2;
    
    /* Level Area */
    fseek(LevelFile, 0x004e, SEEK_SET);
    xScreenCutLeft = ReadValue(LevelFile, 2, 'l');
    yScreenCutTop = ReadValue(LevelFile, 2, 'l');
    xScreenExtensionRight = ReadValue(LevelFile, 2, 'l');
    yScreenExtensionBottom = ReadValue(LevelFile, 2, 'l');

    /* Map Data */
    fseek(LevelFile, 0x0074, SEEK_SET);
    for(int i=0; i<1971; i++)
    {
        if(ReadValue(LevelFile, 2, 'b')) LevelMapData[i].Background=true;
        LevelMapData[i].SpriteID = ReadValue(LevelFile, 2, 'b');
    }
    
    /* Object Data */
    GetSectionOffset(LevelFile, L2BO);
    fseek(LevelFile, 0x08, SEEK_CUR);
    SObjectInfo* ObjectInfo;
    for(int i=0; i<64; i++)
    {
        unsigned char byte = fgetc(LevelFile);
        if(byte != 0xff)
        {
            fseek(LevelFile, -1, SEEK_CUR);
            ObjectInfo = new SObjectInfo;
            ObjectInfo->ID = ReadValue(LevelFile, 2, 'l');
            ObjectInfo->xPos=ReadValue(LevelFile, 2, 'l');
            ObjectInfo->yPos=ReadValue(LevelFile, 2, 'l');
            ObjectInfo->xExt=ReadValue(LevelFile, 2, 'l');
            ObjectInfo->yExt=ReadValue(LevelFile, 2, 'l');
            LevelData->ObjectInfoList.push_back(ObjectInfo);
        }
        else fseek(LevelFile, 0x09, SEEK_CUR);
    }
}

void CLevelData::WriteLevelData()
{
    //unsigned char charBuffer[2];
    //unsigned char byte;

    rewind(LevelFile);

    /* Level Arrangement */
    fseek(LevelFile, 0x0068, SEEK_SET);
    //fputc(LevelFile, LevelArrangementValue);
    LevSpritesPerLine = LevelArrangementList[LevelArrangementValue]+2;
    
    /* Level Area */
    /*fseek(LevelFile, 0x004e, SEEK_SET);
    
    /* Level Area */
    fseek(LevelFile, 0x004e, SEEK_SET);
    WriteValue(xScreenCutLeft, LevelFile, 2, 'l');
    WriteValue(yScreenCutTop, LevelFile, 2, 'l');
    WriteValue(xScreenExtensionRight, LevelFile, 2, 'l');
    WriteValue(yScreenExtensionBottom, LevelFile, 2, 'l');

    /* Map Data */
    int i;
    fseek(LevelFile, 0x0074, SEEK_SET);
    for(i=0; i<1971; i++)
    {
        if(LevelMapData[i].Background) WriteValue(0x4000, LevelFile, 2, 'b');
        else WriteValue(0x0000, LevelFile, 2, 'b');
        WriteValue(LevelMapData[i].SpriteID, LevelFile, 2, 'b');
    }
    
    /* Object Data */
    GetSectionOffset(LevelFile, L2BO);
    fseek(LevelFile, 0x08, SEEK_CUR);
    for(i=0; i<64; i++)
    {
        if(i < ObjectInfoList.size())
        {
            WriteValue(ObjectInfoList[i]->ID, LevelFile, 2, 'l');
            WriteValue(ObjectInfoList[i]->xPos, LevelFile, 2, 'l');
            WriteValue(ObjectInfoList[i]->yPos, LevelFile, 2, 'l');
            WriteValue(ObjectInfoList[i]->xExt, LevelFile, 2, 'l');
            WriteValue(ObjectInfoList[i]->yExt, LevelFile, 2, 'l');
        }
        else for(int j=0; j<10; j++) fputc(0xff, LevelFile);
    }
}

void CLevelData::SpriteToMap(int x, int y, int ID, bool bg)
{
    if(x<0) return;
    if(y<0) return;
    if(x>=LevSpritesPerLine) return;
    if(LevSpritesPerLine*y + x > 1969) return;
    LevelMapData[LevSpritesPerLine*y + x].SpriteID = ID;
    LevelMapData[LevSpritesPerLine*y + x].Background = bg;
}

int CLevelData::SpriteFromMap(int x, int y)
{
    if(x<0) return 0;
    if(y<0) return 0;
    if(x>=LevSpritesPerLine) return 0;
    if(LevSpritesPerLine*y + x > 1969) return 0;
    return LevelMapData[LevSpritesPerLine*y + x].SpriteID;
}

void CLevelData::TileToMap(int x, int y, int ID, bool bg)
{
    int xOff=0, yOff=0;
    for(yOff=0; yOff<TileData->TileList[ID]->ySize; yOff++)
        for(xOff=0; xOff<TileData->TileList[ID]->xSize; xOff++)
            SpriteToMap(x+xOff, y+yOff, TileData->TileList[ID]->SpriteList[yOff * TileData->TileList[ID]->xSize + xOff], bg);
}

int CLevelData::ObjectFromMap(int x, int y)
{
    for(int i=0; i < ObjectInfoList.size(); i++)
    {
        if(CheckObjectUnderCursor(x, y, i)) return i;
    }
    return -1;
}

bool CLevelData::CheckObjectUnderCursor(int x, int y, int oID)
{
    int xPos = ObjectInfoList[oID]->xPos;
    int yPos = ObjectInfoList[oID]->yPos;
    int xExt = ObjectInfoList[oID]->xExt;
    int yExt = ObjectInfoList[oID]->yExt;
    int ID = ObjectInfoList[oID]->ID;

    if( (x > xPos + ObjectData->ObjectList[ID]->xMin)
     && (x < xPos + ObjectData->ObjectList[ID]->xMax + xExt*ObjectData->ObjectList[ID]->xPlus - 1)
     && (y > yPos + ObjectData->ObjectList[ID]->yMin)
     && (y < yPos + ObjectData->ObjectList[ID]->yMax + yExt*ObjectData->ObjectList[ID]->yPlus - 1) ) return true;
    else return false;
}

//SLMapData SpriteFromMap
//DragObject()

#endif
