#ifndef L2WDMAIN
#define L2WDMAIN

#include "L2WMain.h"
#include <GL/freeglut.h>

const unsigned char ModeText[5][7]={"NODRAW", "SPRITE", "TILE  ", "OBJECT", "SELECT"};

CWinMain::CWinMain(int w, int h, int x, int y)  //initialize
{
    glutInitWindowSize(w, h);
    glutInitWindowPosition(x, y);
    MAIN_WIN = glutCreateWindow("PCL2ED v0.1 - Main Window");
    glMatrixMode(GL_PROJECTION);
    glLoadIdentity();
	gluOrtho2D(0, w, h, 0);
    glClearColor(0.0, 0.0, 0.0, 1.0);
    
    Mode=MODE_NODRAW;
    xShift=0; yShift=0;
    BackgroundDraw = false;
    
    glutDisplayFunc(Display);
    glutKeyboardFunc(KeyProcessing);
    glutSpecialFunc(SpecialKeyProcessing);
    glutReshapeFunc(Reshape);
    glutMouseFunc(Mouse);
    glutMotionFunc(MouseMovement);
}

void CWinMain::Display()
{
    glClear(GL_COLOR_BUFFER_BIT);
    
    int x=0, y=0, BorderMode;
    for(int i=0; i<1969; i++)
    {
        SpriteData->DrawSprite(16*x, 8*y, LevelData->LevelMapData[i].SpriteID, LevelData->LevelMapData[i].Background);
        x++;
        if(x>=LevelData->LevSpritesPerLine) {y++; x=0;}
    }
    
    for(int i=0; i<LevelData->ObjectInfoList.size(); i++)
    {
        if(i == MainWindow->SelectedObjectID && MainWindow->Mode == MODE_OBJECT) BorderMode = BORDER_SPECIAL;
        else BorderMode = BORDER_NORMAL;
        ObjectData->DrawObject(LevelData->ObjectInfoList[i]->xPos, 
                               LevelData->ObjectInfoList[i]->yPos,
                               LevelData->ObjectInfoList[i]->xExt,
                               LevelData->ObjectInfoList[i]->yExt,
                               LevelData->ObjectInfoList[i]->ID, BorderMode);
    }
    
    DrawMode();

    MainWindow->DrawLevelArea();
    //DrawSelectionArea;

    glFlush();
}

//map coords
void CWinMain::DrawMapSect(int x1, int y1, int x2, int y2) {
    int i;
    for(int y=y1; y<y2; y++)
    {
        for(int x=x1; x<x2; x++)
        {
            i = y*LevelData->LevSpritesPerLine + x;
            if(i > 1968) return;
            SpriteData->DrawSprite(16*x, 8*y, LevelData->LevelMapData[i].SpriteID, LevelData->LevelMapData[i].Background);
        }
    }
}

void CWinMain::ClearDrawObject(int xPos, int yPos, int xExtension, int yExtension, int ID) {
    int oID = LevelData->ObjectInfoList[ID]->ID;
    CObjectSingle *o = ObjectData->ObjectList[oID];
    CWinMain::DrawMapSect((xPos+o->xMin)/16, (yPos+o->yMin)/8, (xPos+o->xMax+xExtension*o->xPlus-1)/16, (yPos+o->yMax+yExtension*o->yPlus-1)/8);
}

void CWinMain::Reshape(int w, int h)
{
    glViewport(0, 0, w, h); //*MainWindow->Zoom
    glMatrixMode(GL_PROJECTION);
    glLoadIdentity();
    gluOrtho2D(0, w, h, 0);
    glTranslated(16*MainWindow->xShift, 8*MainWindow->yShift, 0);
}

void CWinMain::KeyProcessing(unsigned char key, int x, int y)
{
    switch(MainWindow->Mode)
    {
        case MODE_OBJECT:
            switch(key)
            {
                case 0x7F: //delete key
                    if(MainWindow->SelectedObjectID != -1)
                    {
                        //ClearDrawCurrentObject();
                        LevelData->ObjectInfoList.erase(LevelData->ObjectInfoList.begin() + MainWindow->SelectedObjectID);
                        MainWindow->SelectedObjectID = -1;
                        SetMode(MODE_NODRAW);
                    }
                    break;
                case 'a':
                    if(LevelData->ObjectInfoList[MainWindow->SelectedObjectID]->xExt > 0)
                        LevelData->ObjectInfoList[MainWindow->SelectedObjectID]->xExt--;
                    break;
                case 'd':
                    if(LevelData->ObjectInfoList[MainWindow->SelectedObjectID]->xExt < 63)
                        LevelData->ObjectInfoList[MainWindow->SelectedObjectID]->xExt++;
                    break;
                case 'w':
                    if(LevelData->ObjectInfoList[MainWindow->SelectedObjectID]->yExt > 0)
                        LevelData->ObjectInfoList[MainWindow->SelectedObjectID]->yExt--;
                    break;
                case 's':
                    if(LevelData->ObjectInfoList[MainWindow->SelectedObjectID]->yExt < 63)
                        LevelData->ObjectInfoList[MainWindow->SelectedObjectID]->yExt++;
                    break;
                case '-':
                    if(LevelData->ObjectInfoList[MainWindow->SelectedObjectID]->ID > 0)
                        LevelData->ObjectInfoList[MainWindow->SelectedObjectID]->ID--;
                    break;
                case '+':
                    if(LevelData->ObjectInfoList[MainWindow->SelectedObjectID]->ID < ObjectData->ObjectAmount - 1)
                        LevelData->ObjectInfoList[MainWindow->SelectedObjectID]->ID++;
                    break;
            }
            break;
        case MODE_NODRAW:
            switch(key)
            {
                case '+':
                    if(LevelData->LevelArrangementValue < 6) LevelData->LevelArrangementValue++;
                    LevelData->LevSpritesPerLine = LevelArrangementList[LevelData->LevelArrangementValue] + 2;
                    break;
                case '-':
                    if(LevelData->LevelArrangementValue > 0) LevelData->LevelArrangementValue--;
                    LevelData->LevSpritesPerLine = LevelArrangementList[LevelData->LevelArrangementValue] + 2;
                    break;
            }
            break;
        //default:
            //if(key == 'b') MainWindow->BackgroundDraw = !MainWindow->BackgroundDraw;
            //break;
    }
    if(key == 'b') MainWindow->BackgroundDraw = !MainWindow->BackgroundDraw;
    glutPostRedisplay();
}

void CWinMain::SpecialKeyProcessing(int key, int x, int y)
{
    if(key == GLUT_KEY_F3)
    {
        LevelData->WriteLevelData();
        glRasterPos2i(4 - 16*MainWindow->xShift, 29 - 8*MainWindow->yShift);
        unsigned char text[12] = "LEVEL SAVED";
        glutBitmapString(GLUT_BITMAP_8_BY_13, text);
        glFlush();
        return;
    }
    switch(MainWindow->Mode)
    {
        case MODE_OBJECT:
            switch(key)
            {
                case GLUT_KEY_RIGHT:
                    LevelData->ObjectInfoList[MainWindow->SelectedObjectID]->xPos += 16;
                    break;
                case GLUT_KEY_LEFT:
                    LevelData->ObjectInfoList[MainWindow->SelectedObjectID]->xPos -= 16;
                    break;
                case GLUT_KEY_DOWN:
                    LevelData->ObjectInfoList[MainWindow->SelectedObjectID]->yPos += 8;
                    break;
                case GLUT_KEY_UP:
                    LevelData->ObjectInfoList[MainWindow->SelectedObjectID]->yPos -= 8;
                    break;
            }
            break;
        default:
            switch(key)
            {
                case GLUT_KEY_RIGHT:
                    MainWindow->xShift--;
                    glTranslated(-16, 0, 0); break;
                case GLUT_KEY_LEFT:
                    MainWindow->xShift++;
                    glTranslated(16, 0, 0); break;
                case GLUT_KEY_DOWN:
                    MainWindow->yShift--;
                    glTranslated(0, -8, 0); break;
                case GLUT_KEY_UP:
                    MainWindow->yShift++;
                    glTranslated(0, 8, 0); break;
            }
            break;
    }
    glutPostRedisplay();
}

void CWinMain::MouseMovement(int x, int y)
{
    x -= 16*MainWindow->xShift;
    y -= 8*MainWindow->yShift;
    MainWindow->MouseX = x;
    MainWindow->MouseY = y;
    MainWindow->MouseToPos();
    switch(MainWindow->Mode)
    {
        case MODE_SPRITE:
            switch(MainWindow->MouseButton)
            {
                case GLUT_LEFT_BUTTON:
                    LevelData->SpriteToMap(MainWindow->PosX, MainWindow->PosY, SpriteWindow->SelectedSprite, MainWindow->BackgroundDraw);
                    SpriteData->DrawSprite(16*MainWindow->PosX, 8*MainWindow->PosY, SpriteWindow->SelectedSprite, MainWindow->BackgroundDraw);
                    glFlush();
                    break;
                default: return;
            }
            break;
        /*case MODE_NODRAW:
            switch(MouseButton)
            {
                case GLUT_LEFT_BUTTON:
                    break;
                case GLUT_RIGHT_BUTTON:
                    break;
            }
            break;*/
    }
        
}

void CWinMain::Mouse(int button, int state, int x, int y)
{
    bool DoubleClick = false;
    x -= 16*MainWindow->xShift;
    y -= 8*MainWindow->yShift;
    MainWindow->MouseX = x;
    MainWindow->MouseY = y;
    MainWindow->MouseToPos();
    MainWindow->MouseButton = button;
    if(state == GLUT_DOWN)
    {
        if( MainWindow->MouseX == MainWindow->MouseXPrevious
         && MainWindow->MouseY == MainWindow->MouseYPrevious
         && MainWindow->MouseButton == MainWindow->MouseButtonPrevious) DoubleClick = true;
        MainWindow->MouseXPrevious = MainWindow->MouseX;
        MainWindow->MouseYPrevious = MainWindow->MouseY;
        MainWindow->MouseButtonPrevious = MainWindow->MouseButton;
        switch(button)
        {
            case GLUT_LEFT_BUTTON:
                switch(MainWindow->Mode)
                {
                    case MODE_SPRITE:
                        LevelData->SpriteToMap(MainWindow->PosX, MainWindow->PosY, SpriteWindow->SelectedSprite, MainWindow->BackgroundDraw);
                        SpriteData->DrawSprite(16*MainWindow->PosX, 8*MainWindow->PosY, SpriteWindow->SelectedSprite, MainWindow->BackgroundDraw);
                        glFlush();
                        break;
                    case MODE_TILE:
                        LevelData->TileToMap(MainWindow->PosX, MainWindow->PosY, TileWindow->SelectedTile, MainWindow->BackgroundDraw);
                        TileData->DrawTile(16*MainWindow->PosX, 8*MainWindow->PosY, TileWindow->SelectedTile, MainWindow->BackgroundDraw);
                        glFlush();
                        break;
                    case MODE_OBJECT:
                        //unmark old object
                        DrawCurrentObject(BORDER_NORMAL);
                        MainWindow->SelectedObjectID = LevelData->ObjectFromMap(MainWindow->MouseX, MainWindow->MouseY);
                        if(MainWindow->SelectedObjectID != -1) {
                            ObjectWindow->SelectedObject = LevelData->ObjectInfoList[MainWindow->SelectedObjectID]->ID;
                            //mark new object
                            DrawCurrentObject(BORDER_SPECIAL);
                        }
                        else if(LevelData->ObjectInfoList.size() < 64) //max amount of objects
                        {
                            MainWindow->SelectedObjectID = LevelData->ObjectInfoList.size();
                            SObjectInfo* ObjectInfo = new SObjectInfo;
                            ObjectInfo->ID = ObjectWindow->SelectedObject;
                            ObjectInfo->xPos=16*(x/16);
                            ObjectInfo->yPos=8*(y/8);
                            ObjectInfo->xExt=0;
                            ObjectInfo->yExt=0;
                            LevelData->ObjectInfoList.push_back(ObjectInfo);

                            //mark new object
                            ObjectData->DrawObject(16*MainWindow->PosX, 8*MainWindow->PosY, 0, 0, ObjectWindow->SelectedObject, BORDER_SPECIAL);
                        } else {
                            SetMode(MODE_NODRAW);
                            MainWindow->SelectedObjectID = 0; //prevent value of -1
                        }
                        glFlush();
                        break;
                    case MODE_NODRAW:
                        MainWindow->SelectedObjectID = LevelData->ObjectFromMap(MainWindow->MouseX, MainWindow->MouseY);
                        if(MainWindow->SelectedObjectID != -1)
                        {
                            ObjectWindow->SelectedObject = LevelData->ObjectInfoList[MainWindow->SelectedObjectID]->ID;
                            //mark new object
                            DrawCurrentObject(BORDER_SPECIAL);
                            SetMode(MODE_OBJECT);
                        }
                        break;
                }
                DrawMode();
                break;
            case GLUT_RIGHT_BUTTON:
                if(!DoubleClick) SetMode(MODE_NODRAW);
                else
                {
                    MainWindow->SelectedObjectID = LevelData->ObjectFromMap(MainWindow->MouseX, MainWindow->MouseY);
                    if(MainWindow->SelectedObjectID != -1)
                    {
                        ObjectWindow->SelectedObject = LevelData->ObjectInfoList[MainWindow->SelectedObjectID]->ID;
                        SetMode(MODE_OBJECT);
                    }
                    else
                    {
                        SpriteWindow->SelectedSprite = LevelData->SpriteFromMap(MainWindow->PosX, MainWindow->PosY);
                        SetMode(MODE_SPRITE);
                    }
                }
                break;
        }
    }
    /*else if(state == GLUT_UP)
    {
        gMouseButton = -1;
        if(gMode == MODE_NODRAW)
        {
            //selection routine
            gSelected.x2 = XPosToTile('g', x);
            gSelected.y2 = YPosToTile('g', y);
            invertSelRect();
            drawRectCoords(XTileToPos('g', gSelected.x1), YTileToPos('g', gSelected.y1), XTileToPos('g', gSelected.x2) + 16*gZoom - 1, YTileToPos('g', gSelected.y2) + 8*gZoom - 1);
            glFlush();
        }
    }*/
}
void CWinMain::DrawCurrentObject(int border_mode) {
    if(MainWindow->SelectedObjectID >= 0 && MainWindow->SelectedObjectID <= LevelData->ObjectInfoList.size()) {
    ObjectData->DrawObject(LevelData->ObjectInfoList[MainWindow->SelectedObjectID]->xPos, LevelData->ObjectInfoList[MainWindow->SelectedObjectID]->yPos, LevelData->ObjectInfoList[MainWindow->SelectedObjectID]->xExt, LevelData->ObjectInfoList[MainWindow->SelectedObjectID]->yExt, LevelData->ObjectInfoList[MainWindow->SelectedObjectID]->ID, border_mode);
        glFlush();
    }
}

void CWinMain::ClearDrawCurrentObject() {
    if(MainWindow->SelectedObjectID >= 0 && MainWindow->SelectedObjectID <= LevelData->ObjectInfoList.size()) {
    MainWindow->ClearDrawObject(LevelData->ObjectInfoList[MainWindow->SelectedObjectID]->xPos, LevelData->ObjectInfoList[MainWindow->SelectedObjectID]->yPos, LevelData->ObjectInfoList[MainWindow->SelectedObjectID]->xExt, LevelData->ObjectInfoList[MainWindow->SelectedObjectID]->yExt, LevelData->ObjectInfoList[MainWindow->SelectedObjectID]->ID);
        glFlush();
    }
}

void CWinMain::MouseToPos()
{
    PosX = MouseX/16;
    PosY = MouseY/8;
}

void CWinMain::DrawLevelArea()
{
    glColor3f(1.0, 0.0, 0.5);
    DrawRectCoords(LevelData->xScreenCutLeft + 0x10,
                   LevelData->yScreenCutTop + 0x10,
                   LevelData->xScreenExtensionRight + 320 + 0x10,
                   LevelData->yScreenExtensionBottom + 160 + 0x10);

}

void CWinMain::DrawMode()
{
    glColor3f(0.0, 0.0, 0.0);
    ClearRect(4-16*MainWindow->xShift, 2-8*MainWindow->yShift, 6*8, 13);
    glColor3f(1.0, 1.0, 1.0);
    glRasterPos2i(4 - 16*MainWindow->xShift, 15 - 8*MainWindow->yShift);
    glutBitmapString(GLUT_BITMAP_8_BY_13, ModeText[MainWindow->Mode]);
}

void CWinMain::SetMode(int modeID) {
    MainWindow->Mode = modeID;
    DrawMode();
    glFlush();
}

#endif
